<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/26/2003 *
 * ===============                              Last change: 11/29/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : mysql-manager.php                                *
 * -------------------------------------------------------------------- *
 * Short description : All MySQL-related functions                      *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Alle MySQL-Relevanten Funktionen                 *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

//
function ADD_MODULE_TITLE($mod) {
	global $cacheArray, $_CONFIG;
	$name = ""; $result = false;

	// Is the script installed?
	if (isBooleanConstantAndTrue('mxchange_installed')) {
		// Check if cache is valid
		if ((GET_EXT_VERSION("cache") >= "0.1.2") && (isset($cacheArray['modules']['module'])) && (in_array($mod, $cacheArray['modules']['module']))) {
			// Load from cache
			$name = $cacheArray['modules']['title'][$mod];

			// Update cache hits
			$_CONFIG['cache_hits']++;
		} else {
			// Load from database
			$result = SQL_QUERY_ESC("SELECT title FROM "._MYSQL_PREFIX."_mod_reg WHERE module='%s' LIMIT 1", array($mod), __FILE__, __LINE__);
			list($name) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
		}
	} // END - if

	// Trim name
	$name = trim($name);

	// Still no luck or empty title?
	if (empty($name)) {
		// No name found
		$name = LANG_UNKNOWN_MODULE." (".$mod.")";
		if (SQL_NUMROWS($result) == 0) {
			// Add module to database
			$dummy = CHECK_MODULE($mod);
		} // END - if
	} // END - if

	// Return name
	return $name;
}

// Check validity of a given module name (no file extension)
function CHECK_MODULE($mod) {
	// We need them now here...
	global $cacheArray, $_CONFIG, $cacheInstance;

	// Filter module name (names with low chars and underlines are fine!)
	$mod = preg_replace("/[^a-z_]/", "", $mod);

	// Check for prefix is a extension...
	$modSplit = explode("_", $mod);
	$extension = ""; $mod_chk = $mod;
	//* DEBUG: */ echo __LINE__."*".count($modSplit)."*/".$mod."*<br />";
	if (count($modSplit) == 2) {
		// Okay, there is a seperator (_) in the name so is the first part a module?
		//* DEBUG: */ echo __LINE__."*".$modSplit[0]."*<br />";
		if (EXT_IS_ACTIVE($modSplit[0])) {
			// The prefix is an extension's name, so let's set it
			$extension = $modSplit[0]; $mod = $modSplit[1];
		}
	}

	// Major error in module registry is the default
	$ret = "major";

	// Check if script is installed if not return a "done" to prevent some errors
	if ((!isBooleanConstantAndTrue('mxchange_installed')) || (isBooleanConstantAndTrue('mxchange_installing')) || (!isBooleanConstantAndTrue('admin_registered'))) return "done";

	// Check if cache is latest version
	$locked = "Y"; $hidden = "N"; $admin = "N"; $mem = "N"; $found = false;
	if ((GET_EXT_VERSION("cache") >= "0.1.2") && (isset($cacheArray['modules']['module'])) && (is_array($cacheArray['modules']['module']))) {
		// Is the module cached?
		if (isset($cacheArray['modules']['locked'][$mod_chk])) {
			// Check cache
			$locked = $cacheArray['modules']['locked'][$mod_chk];
			$hidden = $cacheArray['modules']['hidden'][$mod_chk];
			$admin  = $cacheArray['modules']['admin_only'][$mod_chk];
			$mem    = $cacheArray['modules']['mem_only'][$mod_chk];

			// Update cache hits
			$_CONFIG['cache_hits']++;
			$found = true;
		} else {
			// No, then we have to update it!
			$ret = "cache_miss";
		}
	} else {
		// Check for module in database
		$result = SQL_QUERY_ESC("SELECT locked, hidden, admin_only, mem_only FROM "._MYSQL_PREFIX."_mod_reg WHERE module='%s' LIMIT 1", array($mod_chk), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Read data
			list($locked, $hidden, $admin, $mem) = SQL_FETCHROW($result);
			SQL_FREERESULT($result);
			$found = true;
		}
	}

	// Check returned values against current access permissions
	//
	//  Admin access            ----- Guest access -----           --- Guest   or   member? ---
	if ((IS_ADMIN()) || (($locked == "N") && ($admin == "N") && (($mem == "N") || (IS_MEMBER())))) {
		// If you are admin you are welcome for everything!
		$ret = "done";
	} elseif ($locked == "Y") {
		// Module is locked
		$ret = "locked";
	} elseif (($mem == "Y") && (!IS_MEMBER())) {
		// You have to login first!
		$ret = "mem_only";
	} elseif (($admin == "Y") && (!IS_ADMIN())) {
		// Only the Admin is allowed to enter this module!
		$ret = "admin_only";
	}

	// Still no luck or not found?
	if (($ret == "major") || ($ret == "cache_miss") || (!$found)) {
		//              ----- Legacy module -----                                   ---- Module in base folder  ----                       --- Module with extension's name ---
		if ((FILE_READABLE(sprintf("%sinc/modules/%s.php", PATH, $mod))) || (FILE_READABLE(sprintf("%s%s.php", PATH, $mod))) || (FILE_READABLE(sprintf("%s%s/%s.php", PATH, $extension, $mod)))) {
			// Data is missing so we add it
			if (GET_EXT_VERSION("sql_patches") >= "0.3.6") {
				// Since 0.3.6 we have a has_menu column, this took me a half hour
				// to find a loop here... *sigh*
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_mod_reg
(module, locked, hidden, mem_only, admin_only, has_menu) VALUES
('%s','Y','N','N','N','N')", array($mod_chk), __FILE__, __LINE__);
			} else {
				// Wrong/missing sql_patches!
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_mod_reg
(module, locked, hidden, mem_only, admin_only) VALUES
('%s','Y','N','N','N')", array($mod_chk), __FILE__, __LINE__);
			}

			// Everthing is fine?
			if (SQL_AFFECTEDROWS() < 1) {
				// Something bad happend!
				return "major";
			} // END - if

			// Destroy cache here
			REBUILD_CACHE("mod_reg", "modreg");

			// And reload data
			$ret = CHECK_MODULE($mod_chk);
		} else {
			// Module not found we don't add it to the database
			$ret = "404";
		}
	} // END - if

	// Return the value
	return $ret;
}

// Add menu description pending on given file name (without path!)
function ADD_DESCR($ACC_LVL, $file, $return = false, $output = true) {
	global $DEPTH, $_CONFIG;
	$LINK_ADD = ""; $OUT = ""; $AND = "";
	// First we have to do some analysis...
	if (ereg("action-", $file)) {
		// This is an action file!
		$type = "action";
		$search = substr($file, 7);
		switch ($ACC_LVL)
		{
		case "admin":
			$MOD_CHECK = "admin";
			break;

		case "sponsor":
		case "guest":
		case "member":
			$MOD_CHECK = $GLOBALS['module'];
			break;
		}
		$AND = " AND (what='' OR what IS NULL)";
	} elseif (ereg("what-", $file)) {
		// This is an admin what file!
		$type = "what";
		$search = substr($file, 5);
		$AND = "";
		switch ($ACC_LVL)
		{
		case "admin":
			$MOD_CHECK = "admin";
			break;

		case "guest":
		case "member":
			$MOD_CHECK = $GLOBALS['module'];
			if (!IS_ADMIN()) {
				$AND = " AND visible='Y' AND locked='N'";
			}
			break;
		}
		$dummy = substr($search, 0, -4);
		$AND .= " AND action='".GET_ACTION($ACC_LVL, $dummy)."'";
	} elseif (($ACC_LVL == "sponsor") || ($ACC_LVL == "engine")) {
		// Sponsor / engine menu
		$type = "what";
		$search = $file;
		$MOD_CHECK = $GLOBALS['module'];
		$AND = "";
	} else {
		// Other
		$type = "menu";
		$search = $file;
		$MOD_CHECK = $GLOBALS['module'];
		$AND = "";
	}
	if ((!isset($DEPTH)) && (!$return)) {
		$DEPTH = 0;
		$prefix = "<DIV class=\"you_are_here\">".YOU_ARE_HERE."&nbsp;<STRONG><A class=\"you_are_here\" href=\"".URL."/modules.php?module=".$GLOBALS['module'].$LINK_ADD."\">Home</A></STRONG>";
	} else {
		if (!$return) $DEPTH++;
		$prefix = "";
	}

	$prefix .= "&nbsp;-&gt;&nbsp;";

	// We need to remove .php and the end
	if (substr($search, -4, 4) == ".php") {
		// Remove the .php
		$search = substr($search, 0, -4);
	} // END - i

	// Get the title from menu
	$result = SQL_QUERY_ESC("SELECT title FROM "._MYSQL_PREFIX."_%s_menu WHERE %s='%s' ".$AND." LIMIT 1",
	 array($ACC_LVL, $type, $search), __FILE__, __LINE__);

	// Menu found?
	if (SQL_NUMROWS($result) == 1) {
		// Load title
		list($ret) = SQL_FETCHROW($result);

		// Shall we return it?
		if ($return) {
			// Return title
			return $ret;
		} elseif (((GET_EXT_VERSION("sql_patches") >= "0.2.3") && ($_CONFIG['youre_here'] == "Y")) || ((IS_ADMIN()) && ($MOD_CHECK == "admin"))) {
			// Output HTML code
			$OUT = $prefix."<STRONG><A class=\"you_are_here\" href=\"".URL."/modules.php?module=".$MOD_CHECK."&amp;".$type."=".$search.$LINK_ADD."\">".$ret."</A></STRONG>\n";

			// Can we close the you-are-here navigation?
			//* DEBUG: */ echo __LINE__."*".$type."/".$GLOBALS['what']."*<br />\n";
			//* DEBUG: */ die("<pre>".print_r($_CONFIG, true)."</pre>");
			if (($type == "what") || (($type == "action") && ((!isset($GLOBALS['what'])) || ($GLOBALS['what'] == "overview")))) {
				//* DEBUG: */ echo __LINE__."+".$type."+<br />\n";
				$OUT .= "</div>\n";

				// Extension removeip activated?
				if ((EXT_IS_ACTIVE("removeip")) && (isset($_CONFIG['removeip_'.strtolower($ACC_LVL).'_show'])) && ($_CONFIG['removeip_'.strtolower($ACC_LVL).'_show'] == "Y")) {
					// Add anoymity/privacy infos
					$OUT .= REMOVEIP_ADD_INFOS();
				} // END - if

				// Add line-break tag
				$OUT .= "<br />\n";
				$DEPTH = "0";

				// Handle failed logins here if not in guest
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):type={$type},action={$GLOBALS['action']},what={$GLOBALS['what']},lvl={$ACC_LVL}<br />\n";
				if ((($type == "what") || ($type == "action") && ((!isset($GLOBALS['what'])) || ($GLOBALS['what'] == "overview") || ($GLOBALS['what'] == $_CONFIG['index_home']))) && ($ACC_LVL != "guest") && ((GET_EXT_VERSION("sql_patches") >= "0.4.7") || (GET_EXT_VERSION("admins") >= "0.7.0"))) {
					// Handle failture
					$OUT .= HANDLE_LOGIN_FAILTURES($ACC_LVL);
				} // END - if
			} // END - if
		}
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return or output HTML code?
	if ($output) {
		// Output HTML code here
		OUTPUT_HTML($OUT);
	} else {
		// Return HTML code
		return $OUT;
	}
}
//
function ADD_MENU($MODE, $act, $wht) {
	global $_CONFIG;

	// Init some variables
	$main_cnt = 0;
	$AND = "";
	$main_action = "";
	$sub_what = "";

	if (!VALIDATE_MENU_ACTION($MODE, $act, $wht, true)) return CODE_MENU_NOT_VALID;

	// Non-admin shall not see all menus
	if (!IS_ADMIN()) {
		$AND = "AND visible='Y' AND locked='N'";
	}

	// Load SQL data and add the menu to the output stream...
	$result_main = SQL_QUERY_ESC("SELECT title, action FROM "._MYSQL_PREFIX."_%s_menu WHERE (what='' OR what IS NULL) ".$AND." ORDER BY sort",
	 array($MODE), __FILE__, __LINE__);
	//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
	if (SQL_NUMROWS($result_main) > 0) {
		OUTPUT_HTML("<TABLE border=\"0\" cellspacing=\"0\" cellpadding=\"0\" class=\"".$MODE."_menu\">");
		// There are menus available, so we simply display them... :)
		while (list($main_title, $main_action) = SQL_FETCHROW($result_main)) {
			//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
			// Init variables
			$BLOCK_MODE = false; $act = $main_action;

			// Prepare content
			$content = array(
				'action' => $main_action,
				'title'  => $main_title
			);

			// Load menu header template
			LOAD_TEMPLATE($MODE."_menu_title", false, $content);

			$result_sub = SQL_QUERY_ESC("SELECT title, what FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s' AND what != '' AND what IS NOT NULL ".$AND." ORDER BY sort",
			 array($MODE, $main_action), __FILE__, __LINE__);
			$ctl = SQL_NUMROWS($result_sub);
			if ($ctl > 0) {
				$cnt=0;
				while (list($sub_title, $sub_what) = SQL_FETCHROW($result_sub)) {
					// Init content
					$content = "";

					// Full file name for checking menu
					//* DEBUG: */ echo __LINE__.":!!!!".$sub_what."!!!<br />\n";
					$test_inc = sprintf("%sinc/modules/%s/what-%s.php", PATH, $MODE, $sub_what);
					$test = (FILE_READABLE($test_inc));
					if ($test) {
						if ((!empty($wht)) && (($wht == $sub_what))) {
							$content = "<STRONG>";
						}

						// Navigation link
						$content .= "<A name=\"menu\" class=\"menu_blur\" href=\"".URL."/modules.php?module=".$GLOBALS['module']."&amp;what=".$sub_what.ADD_URL_DATA("")."\" target=\"_self\">";
					} else {
						$content .= "<I>";
					}

					// Menu title
					$content .= $_CONFIG['menu_blur_spacer'].$sub_title;

					if ($test) {
						$content .= "</A>";
					} else {
						$content .= "</I>";
					}

					if ((!empty($wht)) && (($wht == $sub_what))) {
						$content .= "</STRONG>";
					}
					$wht = $sub_what; $cnt++;
					// Prepare array
					$content =  array(
						'menu' => $content,
						'what' => $sub_what
					);

					// Add regular menu row or bottom row?
					if ($cnt < $ctl) {
						LOAD_TEMPLATE($MODE."_menu_row", false, $content);
					} else {
						LOAD_TEMPLATE($MODE."_menu_bottom", false, $content);
					}
				}
			} else {
				// This is a menu block... ;-)
				$BLOCK_MODE = true;
				$INC_BLOCK = sprintf("%sinc/modules/%s/action-%s.php", PATH, $MODE, $main_action);
				if (FILE_READABLE($INC_BLOCK)) {
					// Load include file
					if ((!EXT_IS_ACTIVE($main_action)) || ($main_action == "online")) OUTPUT_HTML("<TR>
  <TD class=\"".$MODE."_menu_whats\">");
					//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
					include ($INC_BLOCK);
					//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
					if ((!EXT_IS_ACTIVE($main_action)) || ($main_action == "online")) OUTPUT_HTML("  </TD>
</TR>");
				}
				//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
			}
			$main_cnt++;
			//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
			if (SQL_NUMROWS($result_main) > $main_cnt) 	OUTPUT_HTML("<TR><TD class=\"".$MODE."_menu_seperator\"></TD></TR>");
		}

		// Free memory
		SQL_FREERESULT($result_main);

		// Close table
		//* DEBUG: */ echo __LINE__."/".$main_cnt."/".$main_action."/".$sub_what.":".$GLOBALS['what']."*<br />\n";
		OUTPUT_HTML("</TABLE>");
	}
}
// This patched function will reduce many SELECT queries for the specified or current admin login
function IS_ADMIN($admin="")
{
	global $cacheArray, $_CONFIG;
	$ret = false; $passCookie = ""; $valPass = "";
	//* DEBUG: */ echo __LINE__."ADMIN:".$admin."<br />";

	// If admin login is not given take current from cookies...
	if ((empty($admin)) && (isSessionVariableSet('admin_login')) && (isSessionVariableSet('admin_md5'))) {
		// Get admin login and password from session/cookies
		$admin = get_session('admin_login');
		$passCookie = get_session('admin_md5');
	}
	//* DEBUG: */ echo __LINE__."ADMIN:".$admin."/".$passCookie."<br />";

	// Search in array for entry
	if (isset($cacheArray['admin_hash'])) {
		// Use cached string
		$valPass = $cacheArray['admin_hash'];
	} elseif ((!empty($passCookie)) && (isset($cacheArray['admins']['password'][$admin])) && (!empty($admin))) {
		// Count cache hits
		$_CONFIG['cache_hits']++;

		// Login data is valid or not?
		$valPass = generatePassString($cacheArray['admins']['password'][$admin]);

		// Cache it away
		$cacheArray['admin_hash'] = $valPass;
	} elseif (!empty($admin)) {
		// Search for admin
		$result = SQL_QUERY_ESC("SELECT HIGH_PRIORITY password FROM "._MYSQL_PREFIX."_admins WHERE login='%s' LIMIT 1",
		 array($admin), __FILE__, __LINE__);

		// Is he admin?
		$passDB = "";
		if (SQL_NUMROWS($result) == 1) {
			// Admin login was found so let's load password from DB
			list($passDB) = SQL_FETCHROW($result);

			// Temporary cache it
			$cacheArray['admins']['password'][$admin] = $passDB;

			// Generate password hash
			$valPass = generatePassString($passDB);
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	}

	if (!empty($valPass)) {
		// Check if password is valid
		//* DEBUG: */ print __FUNCTION__."*".$valPass."/".$passCookie."*<br />\n";
		$ret = (($valPass == $passCookie) || ((strlen($valPass) == 32) && ($valPass == md5($passCookie))) || (($valPass == "*FAILED*") && (!EXT_IS_ACTIVE("cache"))));
	}

	// Return result of comparision
	//* DEBUG: */ if (!$ret) echo __LINE__."OK!<br />";
	return $ret;
}
//
function ADD_MAX_RECEIVE_LIST($MODE, $default="", $return=false)
{
	global $_POST;
	$OUT = "";
	switch ($MODE)
	{
	case "guest":
		// Guests (in the registration form) are not allowed to select 0 mails per day.
		$result = SQL_QUERY("SELECT value, comment FROM "._MYSQL_PREFIX."_max_receive WHERE value > 0 ORDER BY value", __FILE__, __LINE__);
		if (SQL_NUMROWS($result) > 0)
		{
			$OUT = "";
  			while (list($value, $comment) = SQL_FETCHROW($result))
			{
				$OUT .= "      <OPTION value=\"".$value."\"";
				if ($_POST['max_mails'] == $value) $OUT .= " selected=\"selected\"";
				$OUT .= ">".$value." ".PER_DAY;
				if (!empty($comment)) $OUT .= " (".$comment.")";
				$OUT .= "</OPTION>\n";
			}
			define('__MAX_RECEIVE_OPTIONS', $OUT);

			// Free memory
			SQL_FREERESULT($result);
			$OUT = LOAD_TEMPLATE("guest_receive_table", true);
		}
		 else
		{
			// Maybe the admin has to setup some maximum values?
		}
		break;

	case "member":
		// Members are allowed to set to zero mails per day (we will change this soon!)
		$result = SQL_QUERY("SELECT value, comment FROM "._MYSQL_PREFIX."_max_receive ORDER BY value", __FILE__, __LINE__);
		if (SQL_NUMROWS($result) > 0)
		{
			$OUT = "";
  			while (list($value, $comment) = SQL_FETCHROW($result))
			{
				$OUT .= "      <OPTION value=\"".$value."\"";
				if ($default == $value) $OUT .= " selected=\"selected\"";
				$OUT .= ">".$value." ".PER_DAY;
				if (!empty($comment)) $OUT .= " (".$comment.")";
				$OUT .= "</OPTION>\n";
			}
			define('__MAX_RECEIVE_OPTIONS', $OUT);
			SQL_FREERESULT($result);
			$OUT = LOAD_TEMPLATE("member_receive_table", true);
		}
		 else
		{
			// Maybe the admin has to setup some maximum values?
			$OUT = LOAD_TEMPLATE("admin_settings_saved", true, NO_MAX_VALUES);
		}
		break;
	}
	if ($return)
	{
		// Return generated HTML code
		return $OUT;
	}
	 else
	{
		// Output directly (default)
		OUTPUT_HTML($OUT);
	}
}
//
function SEARCH_EMAIL_USERTAB($email)
{
	$ret = false;
	$result = SQL_QUERY_ESC("SELECT userid FROM "._MYSQL_PREFIX."_user_data WHERE email LIKE '{PER}%s{PER}' LIMIT 1", array($email), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) $ret = true;
	SQL_FREERESULT($result);
	return $ret;
}
//
function WHAT_IS_VALID($act, $wht, $type="guest")
{
	if (IS_ADMIN())
	{
		// Everything is valid to the admin :-)
		return true;
	}
	 else
	{
		$ret = false;
		$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s' AND what='%s' AND locked='N' LIMIT 1", array($type, $act, $wht), __FILE__, __LINE__);
		// Is "what" valid?
		if (SQL_NUMROWS($result) == 1) $ret = true;
		SQL_FREERESULT($result);
		return $ret;
	}
}
//
function IS_MEMBER()
{
	global $status, $LAST, $cacheArray;
	if (!is_array($LAST)) $LAST = array();
	$ret = false;

	// is the cache entry there?
	if (isset($cacheArray['is_member'])) {
		// Then return it
		return $cacheArray['is_member'];
	} // END - if

	// Fix "deleted" cookies first
	FIX_DELETED_COOKIES(array('userid','u_hash','lifetime'));

	// Are cookies set?
	if ((!empty($GLOBALS['userid'])) && (isSessionVariableSet('u_hash')) && (isSessionVariableSet('lifetime')) && (defined('COOKIE_PATH')))
	{
		// Cookies are set with values, but are they valid?
		$result = SQL_QUERY_ESC("SELECT password, status, last_module, last_online FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s LIMIT 1",
		 array($GLOBALS['userid']), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1)
		{
			// Load data from cookies
			list($password, $status, $mod, $onl) = SQL_FETCHROW($result);

			// Validate password by created the difference of it and the secret key
			$valPass = generatePassString($password);

			// Transfer last module and online time
			if ((!empty($mod)) && (empty($LAST['module']))) { $LAST['module'] = $mod; $LAST['online'] = $onl; }

			// So did we now have valid data and an unlocked user?
			//* DEBUG: */ echo $valPass."<br />".get_session('u_hash')."<br />";
			if (($status == "CONFIRMED") && ($valPass == get_session('u_hash'))) {
				// Account is confirmed and all cookie data is valid so he is definely logged in! :-)
				$ret = true;
			} else {
				// Maybe got locked etc.
				//* DEBUG: */ echo __LINE__."!!!<br />";
				destroy_user_session();

				// Reset userid
				$GLOBALS['userid'] = 0;
			}
		} else {
			// Cookie data is invalid!
			//* DEBUG: */ echo __LINE__."***<br />";
			destroy_user_session();

			// Reset userid
			$GLOBALS['userid'] = 0;
		}

		// Free memory
		SQL_FREERESULT($result);
	} else {
		// Cookie data is invalid!
		//* DEBUG: */ echo __LINE__."///<br />";
		destroy_user_session();

		// Reset userid
		$GLOBALS['userid'] = 0;
	}

	// Cache status
	$cacheArray['is_member'] = $ret;

	// Return status
	return $ret;
}
//
function UPDATE_LOGIN_DATA () {
	global $LAST, $_CONFIG;
	if (!is_array($LAST)) $LAST = array();

	// Recheck if logged in
	if (!IS_MEMBER()) return false;

	// Secure user ID
	$GLOBALS['userid'] = bigintval(get_session('userid'));

	// Extract last online time (life) and how long is auto-login valid (time)
	$newl = time() + bigintval(get_session('lifetime'));

	// Load last module and last online time
	$result = SQL_QUERY_ESC("SELECT last_module, last_online FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s LIMIT 1", array($GLOBALS['userid']), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) {
		// Load last module and online time
		list($mod, $onl) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);

		// Maybe first login time?
		if (empty($mod)) $mod = "login";

		if (set_session("userid", $GLOBALS['userid'], $newl, COOKIE_PATH) && set_session("u_hash", get_session('u_hash'), $newl, COOKIE_PATH) && set_session("lifetime", bigintval(get_session('lifetime')), $newl, COOKIE_PATH)) {
			// This will be displayed on welcome page! :-)
			if (empty($LAST['module'])) {
				$LAST['module'] = $mod; $LAST['online'] = $onl;
			} // END - if

			// "what" not set?
			if (empty($GLOBALS['what'])) {
				// Fix it to default
			 	$GLOBALS['what'] = "welcome";
			 	if (!empty($_CONFIG['index_home'])) $GLOBALS['what'] = $_CONFIG['index_home'];
			} // END - if

			// Update last module / online time
			$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_user_data SET last_module='%s', last_online=UNIX_TIMESTAMP(), REMOTE_ADDR='%s' WHERE userid=%s LIMIT 1",
			 array($GLOBALS['what'], GET_REMOTE_ADDR(), $GLOBALS['userid']), __FILE__, __LINE__);
		}
	}  else {
		// Destroy session, we cannot update!
		destroy_user_session();
	}
}
//
function VALIDATE_MENU_ACTION ($MODE, $act, $wht, $UPDATE=false)
{
	$ret = false;
	$ADD = "";
	if ((!IS_ADMIN()) && ($MODE != "admin")) $ADD = " AND locked='N'";
	//* DEBUG: */ echo __LINE__.":".$MODE."/".$act."/".$wht."*<br />\n";
	if (($MODE != "admin") && ($UPDATE))
	{
		// Update guest or member menu
		$SQL = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_%s_menu SET counter=counter+1 WHERE action='%s' AND what='%s'".$ADD." LIMIT 1",
		 array($MODE, $act, $wht), __FILE__, __LINE__, false);
	}
	 elseif ($wht != "overview")
	{
		// Other actions
		$SQL = SQL_QUERY_ESC("SELECT id, what FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s'".$ADD." ORDER BY action DESC LIMIT 1",
		 array($MODE, $act), __FILE__, __LINE__, false);
	}
	 else
	{
		// Admin login overview
		$SQL = SQL_QUERY_ESC("SELECT id, what FROM "._MYSQL_PREFIX."_%s_menu WHERE action='%s' AND (what='' OR what IS NULL)".$ADD." ORDER BY action DESC LIMIT 1",
		 array($MODE, $act), __FILE__, __LINE__, false);
	}

	// Run SQL command
	$result = SQL_QUERY($SQL, __FILE__, __LINE__);
	if ($UPDATE)
	{
		if (SQL_AFFECTEDROWS() == 1) $ret = true;
		//* DEBUG: */ debug_print_backtrace();
	}
	 else
	{
		if (SQL_NUMROWS($result) == 1) {
			list($id, $wht2) = SQL_FETCHROW($result);
			//* DEBUG: */ echo __LINE__."+".$SQL."+<br />\n";
			//* DEBUG: */ echo __LINE__."*".$id."/".$wht."/".$wht2."*<br />\n";
			$ret = true;
		}
	}

	// Free memory
	SQL_FREERESULT($result);

	// Return result
	return $ret;
}
//
function GET_MOD_DESCR($MODE, $wht, $column="what")
{
	// Fix empty "what"
	if (empty($wht)) {
		$wht = "welcome";
		if (!empty($_CONFIG['index_home'])) $wht = $_CONFIG['index_home'];
	} // END - if

	// Default is not found
	$ret = "??? (".$wht.")";

	// Look for title
	$result = SQL_QUERY_ESC("SELECT title FROM "._MYSQL_PREFIX."_%s_menu WHERE %s='%s' LIMIT 1",
		array($MODE, $column, $wht), __FILE__, __LINE__);

	// Is there an entry?
	if (SQL_NUMROWS($result) == 1) {
		// Fetch the title
		list($ret) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);
	return $ret;
}
//
function SEND_MODE_MAILS($mod, $modes)
{
	global $_CONFIG, $DATA;

	// Load hash
	$result_main = SQL_QUERY_ESC("SELECT password FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s AND status='CONFIRMED' LIMIT 1",
	 array($GLOBALS['userid']), __FILE__, __LINE__);
	if (SQL_NUMROWS($result_main) == 1) {
		// Load hash from database
		list($hashDB) = SQL_FETCHROW($result_main);

		// Extract salt from cookie
		$salt = substr(get_session('u_hash'), 0, -40);

		// Now let's compare passwords
		$hash = generatePassString($hashDB);
		if (($hash == get_session('u_hash')) || ($_POST['pass1'] == $_POST['pass2'])) {
			// Load user's data
			$result = SQL_QUERY_ESC("SELECT gender, surname, family, street_nr, country, zip, city, email FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s AND password='%s' LIMIT 1",
			 array($GLOBALS['userid'], $hashDB), __FILE__, __LINE__);
			if (SQL_NUMROWS($result) == 1) {
				// Load the data
				$DATA = SQL_FETCHROW($result);

				// Free result
				SQL_FREERESULT($result);

				// Translate gender
				$DATA[0] = TRANSLATE_GENDER($DATA[0]);

				// Clear/init the content variable
				$content = "";
				$DATA['info'] = "";

				switch ($mod)
				{
				case "mydata":
					foreach ($modes as $mode) {
						switch ($mode)
						{
						case "normal": break; // Do not add any special lines

						case "email": // Email was changed!
							$content = MEMBER_CHANGED_EMAIL.": ".$_POST['old_addy']."\n";
							break;

						case "pass": // Password was changed
							$content = MEMBER_CHANGED_PASS."\n";
							break;

						default:
							$content = MEMBER_UNKNOWN_MODE.": ".$mode."\n\n";
							break;
						}
					} // END - if

					if (EXT_IS_ACTIVE("country")) {
						// Replace code with description
						$DATA[4] = COUNTRY_GENERATE_INFO($_POST['country_code']);
					}

					// Load template
					$msg = LOAD_EMAIL_TEMPLATE("member_mydata_notify", $content, $GLOBALS['userid']);

					if ($_CONFIG['admin_notify'] == "Y") {
						// The admin needs to be notified about a profile change
						$msg_admin = "admin_mydata_notify";
						$sub_adm = ADMIN_CHANGED_DATA;
					} else {
						// No mail to admin
						$msg_admin = "";
						$sub_adm = "";
					}

					// Set subject lines
					$sub_mem = MEMBER_CHANGED_DATA;

					// Output success message
					$content = "<STRONG><SPAN class=\"member_done\">".MYDATA_MAIL_SENT."</SPAN></STRONG>";
					break;

				default:
					$content = "<STRONG><SPAN class=\"member_failed\">".UNKNOWN_MODULE."</SPAN></STRONG>";
					break;
				}
			} else {
				// Could not load profile data
				$content = "<STRONG><SPAN class=\"member_failed\">".MEMBER_CANNOT_LOAD_PROFILE."</SPAN></STRONG>";
			}
		} else {
			// Passwords mismatch
			$content = "<STRONG><SPAN class=\"member_failed\">".MEMBER_PASSWORD_ERROR."</SPAN></STRONG>";
		}
	} else {
		// Could not load profile
		$content = "<STRONG><SPAN class=\"member_failed\">".MEMBER_CANNOT_LOAD_PROFILE."</SPAN></STRONG>";
	}

	// Send email to user if required
	if ((!empty($sub_mem)) && (!empty($msg))) {
		// Send member mail
		SEND_EMAIL($DATA[7], $sub_mem, $msg);
	}

	// Send only if no other error has occured
	if (empty($content)) {
		if ((!empty($sub_adm)) && (!empty($msg_admin))) {
			// Send admin mail
			SEND_ADMIN_NOTIFICATION($sub_adm, $msg_admin, $content, $GLOBALS['userid']);
		} elseif ($_CONFIG['admin_notify'] == "Y") {
			// Cannot send mails to admin!
			$content = CANNOT_SEND_ADMIN_MAILS;
		} else {
			// No mail to admin
			$content = "<STRONG><SPAN class=\"member_done\">".MYDATA_MAIL_SENT."</SPAN></STRONG>";
		}
	}

	// Load template
	LOAD_TEMPLATE("admin_settings_saved", false, $content);
}
// Update module counter
function COUNT_MODULE($mod)
{
	if ($mod != "css")
	{
		// Do count all other modules but not accesses on CSS file css.php!
		$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_mod_reg SET clicks=clicks+1 WHERE module='%s' LIMIT 1",
		 array($mod), __FILE__, __LINE__);
	}
}
// Get action value from mode (admin/guest/member) and what-value
function GET_ACTION ($MODE, &$wht)
{
	global $ret, $_CONFIG;
	// @DEPRECATED Init status
	$ret = "";

	//* DEBUG: */ echo __LINE__."=".$MODE."/".$wht."/".$GLOBALS['action']."=<br />";
	if ((empty($wht)) && ($MODE != "admin")) {
		$wht = "welcome";
		if (!empty($_CONFIG['index_home'])) $wht = $_CONFIG['index_home'];
	} // END - if

	if ($MODE == "admin") {
		// Action value for admin area
		if (!empty($GLOBALS['action'])) {
			// Get it directly from URL
			return $GLOBALS['action'];
		} elseif (($wht == "overview") || (empty($GLOBALS['what']))) {
			// Default value for admin area
			$ret = "login";
		}
	} elseif (!empty($GLOBALS['action'])) {
		// Get it directly from URL
		return $GLOBALS['action'];
	}
	//* DEBUG: */ echo __LINE__."*".$ret."*<br />\n";

	if (MODULE_HAS_MENU($MODE)) {
		// Rewriting modules to menu
		switch ($MODE) {
			case "index": $MODE = "guest";  break;
			case "login": $MODE = "member"; break;
		} // END - switch

		// Guest and member menu is "main" as the default
		if (empty($ret)) $ret = "main";

		// Load from database
		$result = SQL_QUERY_ESC("SELECT action FROM "._MYSQL_PREFIX."_%s_menu WHERE what='%s' LIMIT 1",
		 array($MODE, $wht), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Load action value and pray that this one is the right you want... ;-)
			list($ret) = SQL_FETCHROW($result);
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	} // END - if

	// Return action value
	return $ret;
}
//
function GET_CATEGORY ($cid) {
	// Default is not found
	$ret = _CATEGORY_404;

	// Is the category id set?
	if ($cid == "0") {
		// No category
		$ret = _CATEGORY_NONE;
	} elseif ($cid > 0) {
		// Lookup the category in database
		$result = SQL_QUERY_ESC("SELECT cat FROM "._MYSQL_PREFIX."_cats WHERE id=%s LIMIT 1",
			array(bigintval($cid)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Category found... :-)
			list($ret) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	} // END - if

	// Return result
	return $ret;
}
//
function GET_PAYMENT ($pid, $full=false) {
	// Default is not found
	$ret = _PAYMENT_404;

	// Load payment data
	$result = SQL_QUERY_ESC("SELECT mail_title, price FROM "._MYSQL_PREFIX."_payments WHERE id=%s LIMIT 1",
		array(bigintval($pid)), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) {
		// Payment type found... :-)
		if (!$full) {
			// Return only title
			list($ret) = SQL_FETCHROW($result);
		} else {
			// Return title and price
			list($t, $p) = SQL_FETCHROW($result);
			$ret = $t." / ".TRANSLATE_COMMA($p)." ".POINTS;
		}
	}

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $ret;
}
//
function GET_PAY_POINTS($pid, $lookFor="price")
{
	$ret = "-1";
	$result = SQL_QUERY_ESC("SELECT %s FROM "._MYSQL_PREFIX."_payments WHERE id=%s LIMIT 1",
		array($lookFor, $pid), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1)
	{
		// Payment type found... :-)
		list($ret) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);
	}
	return $ret;
}
// Remove a receiver's ID from $ARRAY and add a link for him to confirm
function REMOVE_RECEIVER(&$ARRAY, $key, $uid, $pool_id, $stats_id="", $bonus=false)
{
	$ret = "failed";
	if ($uid > 0)
	{
		// Remove entry from array
		unset($ARRAY[$key]);

		// Is there already a line for this user available?
		if ($stats_id > 0)
		{
			// Only when we got a real stats ID continue searching for the entry
			$type = "NORMAL"; $rowName = "stats_id";
			if ($bonus) { $type = "BONUS"; $rowName = "bonus_id"; }
			$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_user_links WHERE %s='%s' AND userid=%s AND link_type='%s' LIMIT 1",
			 array($rowName, $stats_id, bigintval($uid), $type), __FILE__, __LINE__);
			if (SQL_NUMROWS($result) == 0)
			{
				// No, so we add one!
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_user_links (%s, userid, link_type) VALUES ('%s','%s','%s')",
				 array($rowName, $stats_id, bigintval($uid), $type), __FILE__, __LINE__);
				$ret = "done";
			}
			 else
			{
				// Already found
				$ret = "already";
			}

			// Free memory
			SQL_FREERESULT($result);
		}
	}
	// Return status for sending routine
	return $ret;
}
//
function GET_TOTAL_DATA($search, $tableName, $lookFor, $whereStatement="userid", $onlyRows=false) {
	$ret = 0;
	if ($onlyRows) {
		// Count rows
		$result = SQL_QUERY_ESC("SELECT COUNT(%s) FROM "._MYSQL_PREFIX."_%s WHERE %s='%s'",
		 array($lookFor, $tableName, $whereStatement, $search), __FILE__, __LINE__);
	} else {
		// Add all rows
		$result = SQL_QUERY_ESC("SELECT SUM(%s) FROM "._MYSQL_PREFIX."_%s WHERE %s='%s'",
		 array($lookFor, $tableName, $whereStatement, $search), __FILE__, __LINE__);
	}

	// Load row
	list($ret) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Fix empty values
	if ((empty($ret)) && ($lookFor != "counter") && ($lookFor != "id") && ($lookFor != "userid")) {
		$ret = "0.00000";
	} // END - if

	// Return value
	return $ret;
}
// "Getter fro ref level percents
function GET_REF_LEVEL_PERCENTS ($level) {
	global $cacheInstance, $_CONFIG, $cacheArray;

	// Default is zero
	$per = 0;

	// Do we have cache?
	if ((isset($cacheArray['ref_depths']['level'])) && (EXT_IS_ACTIVE("cache"))) {
		// First look for level
		$key = array_search($level, $cacheArray['ref_depths']['level']);
		if ($key !== false) {
			// Entry found!
			$per = $cacheArray['ref_depths']['percents'][$key];

			// Count cache hit
			$_CONFIG['cache_hits']++;
		}
	} else {
		// Get referal data
		$result_lvl = SQL_QUERY_ESC("SELECT percents FROM "._MYSQL_PREFIX."_refdepths WHERE level='%s' LIMIT 1",
			array(bigintval($level)), __FILE__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result_lvl) == 1) {
			// Get percents
			list($per) = SQL_FETCHROW($result_lvl);
		} // END - if

		// Free result
		SQL_FREERESULT($result_lvl);
	}

	// Return percent
	return $per;
}
/**
 *
 * Dynamic referral system, can also send mails!
 *
 * uid         = Referral ID wich should receive...
 * points      = ... xxx points
 * send_notify = shall I send the referral an email or not?
 * rid         = inc/modules/guest/what-confirm.php need this (DEPRECATED???)
 * locked      = Shall I pay it to normal (false) or locked (true) points ammount?
 * add_mode    = Add points only to $uid or also refs? (WARNING! Changing "ref" to "direct"
 *               for default value will cause no referral will get points ever!!!)
 */
function ADD_POINTS_REFSYSTEM($uid, $points, $send_notify=false, $rid="0", $locked=false, $add_mode="ref") {
	//* DEBUG: */ print "----------------------- <font color=\"#00aa00\">".__FUNCTION__." - ENTRY</font> ------------------------<ul><li>\n";
	global $DEPTH, $_CONFIG, $DATA, $cacheArray;

	// Convert mode to lower-case
	$add_mode = strtolower($add_mode);

	// When $uid = 0 add points to jackpot
	if ($uid == "0") {
		// Add points to jackpot
		ADD_JACKPOT($points);
		return;
	} // END - if

	// Count up referral depth
	if (!isset($DEPTH)) {
		// Initialialize referral system
		$DEPTH = 0;
	} else {
		// Increase referral level
		$DEPTH++;
	}

	// Default is "normal" points
	$data = "points";

	// Which points, locked or normal?
	if ($locked) $data = "locked_points";

	// Check user account
	$result_user = SQL_QUERY_ESC("SELECT refid, email FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s AND status='CONFIRMED' LIMIT 1",
	 array(bigintval($uid)), __FILE__, __LINE__);

	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},numRows=".SQL_NUMROWS($result_user).",points={$points}<br />\n";
	if (SQL_NUMROWS($result_user) == 1) {
		// This is the user and his ref
		list($ref, $email) = SQL_FETCHROW($result_user);
		$cacheArray['add_uid'][$ref] = $uid;

		// Get percents
		$per = GET_REF_LEVEL_PERCENTS($DEPTH);
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},points={$points},depth={$DEPTH},per={$per},mode={$add_mode}<br />\n";

		// Some percents found?
		if ($per > 0) {
			// Calculate new points
			//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},points={$points},per={$per},depth={$DEPTH}<br />\n";
			$ref_points = $points * $per / 100;

			// Pay refback here if level > 0 and in ref-mode
			if ((EXT_IS_ACTIVE("refback")) && ($DEPTH > 0) && ($per < 100) && ($add_mode == "ref") && (isset($cacheArray['add_uid'][$uid]))) {
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},data={$cacheArray['add_uid'][$uid]},ref_points={$ref_points},depth={$DEPTH} - BEFORE!<br />\n";
				$ref_points = ADD_REFBACK_POINTS($cacheArray['add_uid'][$uid], $uid, $points, $ref_points);
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},data={$cacheArray['add_uid'][$uid]},ref_points={$ref_points},depth={$DEPTH} - AFTER!<br />\n";
			} // END - if

			// Update points...
			SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_user_points SET %s=%s+%s WHERE userid=%s AND ref_depth='%s' LIMIT 1",
			 array($data, $data, $ref_points, bigintval($uid), bigintval($DEPTH)), __FILE__, __LINE__);
			//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):data={$data},ref_points={$ref_points},uid={$uid},depth={$DEPTH},mode={$add_mode} - UPDATE! (".SQL_AFFECTEDROWS().")<br />\n";

			// No entry updated?
			if (SQL_AFFECTEDROWS() < 1) {
				// First ref in this level! :-)
				$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_user_points (userid,ref_depth,%s) VALUES (%s,%s,%s)",
				 array($data, bigintval($uid), bigintval($DEPTH), $ref_points), __FILE__, __LINE__);
			} // END - if

			// Update mediadata as well
			if (GET_EXT_VERSION("mediadata") >= "0.0.4") {
				// Update database
				MEDIA_UPDATE_ENTRY(array("total_points"), "add", $ref_points);
			} // END - if

			// Points updated, maybe I shall send him an email?
			if (($send_notify) && ($ref > 0) && (!$locked)) {
				// Prepare content
				$content = array(
					'percent' => $per,
					'level'   => bigintval($DEPTH),
					'points'  => $ref_points,
					'refid'   => bigintval($ref)
				);

				// Load email template
				$msg = LOAD_EMAIL_TEMPLATE("confirm-referral", $content, bigintval($uid));

				SEND_EMAIL($email, THANX_REFERRAL_ONE, $msg);
			} elseif (($send_notify) && ($ref == 0) && (!$locked) && ($add_mode == "direct") && (!defined('__POINTS_VALUE'))) {
				// Direct payment shall be notified about
				define('__POINTS_VALUE', $ref_points);

				// Prepare content
				$content = array(
					'text'   => REASON_DIRECT_PAYMENT,
					'points' => TRANSLATE_COMMA($ref_points)
				);

				// Load message
				$msg = LOAD_EMAIL_TEMPLATE("add-points", $content, $uid);

				// And sent it away
				SEND_EMAIL($email, SUBJECT_DIRECT_PAYMENT, $msg);
				if (!isset($_GET['mid'])) LOAD_TEMPLATE("admin_settings_saved", false, ADMIN_POINTS_ADDED);
			}

			// Maybe there's another ref?
			if (($ref > 0) && ($points > 0) && ($ref != $uid) && ($add_mode == "ref")) {
				// Then let's credit him here...
				//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},ref={$ref},points={$points} - ADVANCE!<br />\n";
				ADD_POINTS_REFSYSTEM($ref, $points, $send_notify, $ref, $locked);
			} // END - if
		} // END - if
	} // END - if

	// Free result
	SQL_FREERESULT($result_user);
	//* DEBUG: */ print "</li></ul>----------------------- <font color=\"#aa0000\">".__FUNCTION__." - EXIT</font> ------------------------<br />\n";
}
//
function UPDATE_REF_COUNTER ($uid) {
	global $cacheArray, $cacheInstance;

	// Make it sure referral level zero (member him-/herself) is at least selected
	if (empty($cacheArray['ref_level'][$uid])) $cacheArray['ref_level'][$uid] = 1;
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},level={$cacheArray['ref_level'][$uid]}<br />\n";

	// Update counter
	$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_refsystem SET counter=counter+1 WHERE userid=%s AND level='%s' LIMIT 1",
		array(bigintval($uid), $cacheArray['ref_level'][$uid]), __FILE__, __LINE__);

	// When no entry was updated then we have to create it here
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):updated=".SQL_AFFECTEDROWS()."<br />\n";
	if (SQL_AFFECTEDROWS() < 1) {
		// First count!
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_refsystem (userid, level, counter) VALUES (%s,%s,1)",
			array(bigintval($uid), $cacheArray['ref_level'][$uid]), __FILE__, __LINE__);
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid}<br />\n";
	} // END - if

	// Check for his referral
	$result = SQL_QUERY_ESC("SELECT refid FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s LIMIT 1",
		array(bigintval($uid)), __FILE__, __LINE__);

	// Load refid
	list($ref) = SQL_FETCHROW($result);

	// Free memory
	SQL_FREERESULT($result);
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):uid={$uid},ref={$ref}<br />\n";

	// When he has a referral...
	if (($ref > 0) && ($ref != $uid)) {
		// Move to next referral level and count his counter one up!
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):ref={$ref} - ADVANCE!<br />\n";
		$cacheArray['ref_level'][$uid]++; UPDATE_REF_COUNTER($ref);
	} elseif ((($ref == $uid) || ($ref == 0)) && (GET_EXT_VERSION("cache") >= "0.1.2")) {
		// Remove cache here
		//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):ref={$ref} - CACHE!<br />\n";
		if ($cacheInstance->cache_file("refsystem", true)) $cacheInstance->cache_destroy();
	}

	// "Walk" back here
	$cacheArray['ref_level'][$uid]--;

	// Handle refback here if extension is installed
	if (EXT_IS_ACTIVE("refback")) {
		UPDATE_REFBACK_TABLE($uid);
	} // END - if
}
// Updates/extends the online list
function UPDATE_ONLINE_LIST($SID, $mod, $act, $wht) {
	global $_CONFIG;

	// Do not update online list when extension is deactivated
	if (!EXT_IS_ACTIVE("online", true)) return;

	// Empty session?
	if (empty($SID)) {
		// This is invalid here!
		print "Invalid session. Backtrace:<pre>";
		debug_print_backtrace();
		die("</pre>");
	} // END - if

	// Initialize variables
	$uid = 0; $rid = 0; $MEM = "N"; $ADMIN = "N";

	// Valid userid?
	if ((!empty($GLOBALS['userid'])) && ($GLOBALS['userid'] > 0) && (IS_MEMBER())) {
		// Is valid user
		$uid = bigintval($GLOBALS['userid']);
		$MEM = "Y";
	} // END - if

	if (IS_ADMIN()) {
		// Is administrator
		$ADMIN = "Y";
	} // END - if

	if (isSessionVariableSet('up_refid')) {
		// Check cookie
		if (get_session('refid') > 0) $rid = bigintval($GLOBALS['refid']);
	} // END - if

	// Now search for the user
	$result = SQL_QUERY_ESC("SELECT timestamp FROM "._MYSQL_PREFIX."_online
WHERE sid='%s' LIMIT 1",
 array($SID), __FILE__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Then update it
		SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_online SET
module='%s',
action='%s',
what='%s',
userid=%s,
refid=%s,
is_member='%s',
is_admin='%s',
timestamp=UNIX_TIMESTAMP()
WHERE sid='%s' LIMIT 1",
			array($mod, $act, $wht, $uid, $rid, $MEM, $ADMIN, $SID), __FILE__, __LINE__
		);
	} else {
		// No entry does exists so we simply add it!
		SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_online (module, action, what, userid, refid, is_member, is_admin, timestamp, sid, ip) VALUES ('%s','%s','%s', %s, %s, '%s','%s', UNIX_TIMESTAMP(), '%s','%s')",
			array($mod, $act, $wht, $uid, $rid, $MEM, $ADMIN, $SID, GET_REMOTE_ADDR()), __FILE__, __LINE__
		);
	}

	// Free result
	SQL_FREERESULT($result);

	// Purge old entries
	$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_online WHERE timestamp <= (UNIX_TIMESTAMP() - %s)",
	 array($_CONFIG['online_timeout']), __FILE__, __LINE__);
}
// OBSULETE: Sends out mail to all administrators
function SEND_ADMIN_EMAILS($subj, $msg) {
	// Load all admin email addresses
	$result = SQL_QUERY("SELECT email FROM "._MYSQL_PREFIX."_admins ORDER BY id ASC", __FILE__, __LINE__);
	while (list($email) = SQL_FETCHROW($result)) {
		// Send the email out
		SEND_EMAIL($email, $subj, $msg);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Really simple... ;-)
}
// Get ID number from administrator's login name
function GET_ADMIN_ID($login) {
	global $cacheArray, $_CONFIG;
	$ret = "-1";
	if (!empty($cacheArray['admins']['aid'][$login])) {
		// Check cache
		$ret = $cacheArray['admins']['aid'][$login];

		// Update cache hits
		$_CONFIG['cache_hits']++;
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_admins WHERE login='%s' LIMIT 1",
		 array($login), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			list($ret) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	}
	return $ret;
}
//
// Get password hash from administrator's login name
function GET_ADMIN_HASH($login)
{
	global $cacheArray, $_CONFIG;
	$ret = "-1";
	if (!empty($cacheArray['admins']['password'][$login])) {
		// Check cache
		$ret = $cacheArray['admins']['password'][$login];

		// Update cache hits
		$_CONFIG['cache_hits']++;
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT password FROM "._MYSQL_PREFIX."_admins WHERE login='%s' LIMIT 1",
		 array($login), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($ret) = SQL_FETCHROW($result);

			// Set cache
			$cacheArray['admins']['password'][$login] = $ret;
		}

		// Free result
		SQL_FREERESULT($result);
	}
	return $ret;
}
//
function GET_ADMIN_LOGIN ($aid) {
	global $cacheArray, $_CONFIG;
	$ret = "***";
	if (!empty($cacheArray['admins']['login'][$aid])) {
		// Get cache
		$ret = $cacheArray['admins']['login'][$aid];

		// Update cache hits
		$_CONFIG['cache_hits']++;
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result = SQL_QUERY_ESC("SELECT login FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
			array(bigintval($aid)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($ret) = SQL_FETCHROW($result);

			// Set cache
			$cacheArray['admins']['login'][$aid] = $ret;
		} // END - if

		// Free memory
		SQL_FREERESULT($result);
	}
	return $ret;
}
// Get email address of admin id
function GET_ADMIN_EMAIL ($aid) {
	global $cacheArray, $_CONFIG;

	$ret = "***";
	if (!empty($cacheArray['admins']['email'])) {
		// Get cache
		$ret = $cacheArray['admins']['email'][$aid];

		// Update cache hits
		$_CONFIG['cache_hits']++;
	} elseif (!EXT_IS_ACTIVE("cache")) {
		// Load from database
		$result_aid = SQL_QUERY_ESC("SELECT email FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
			array(bigintval($ret)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Get data
			list($ret) = SQL_FETCHROW($result_aid);

			// Set cache
			$cacheArray['admins']['email'][$aid] = $ret;
			} // END - if

		// Free result
		SQL_FREERESULT($result_aid);
	}

	// Return email
	return $ret;
}
// Get default ACL  of admin id
function GET_ADMIN_DEFAULT_ACL ($aid) {
	global $cacheArray, $_CONFIG;

	$ret = "***";
	if (!empty($cacheArray['admins']['def_acl'])) {
		// Use cache
		$ret = $cacheArray['admins']['def_acl'][$aid];

		// Update cache hits
		$_CONFIG['cache_hits']++;
	} else {
		// Load from database
		$result_aid = SQL_QUERY_ESC("SELECT default_acl FROM "._MYSQL_PREFIX."_admins WHERE id=%s LIMIT 1",
			array(bigintval($ret)), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($ret) = SQL_FETCHROW($result_aid);

			// Set cache
			$cacheArray['admins']['def_acl'][$aid] = $ret;
		}

		// Free result
		SQL_FREERESULT($result_aid);
	}

	// Return email
	return $ret;
}
//
function ADD_OPTION_LINES($table, $id, $name, $default="",$special="",$where="") {
	$ret = "";
	if ($table == "/ARRAY/") {
		// Selection from array
		if (is_array($id) && is_array($name) && sizeof($id) == sizeof($name)) {
			// Both are arrays
			foreach ($id as $idx => $value) {
				$ret .= "<OPTION value=\"".$value."\"";
				if ($default == $value) $ret .= " selected checked";
				$ret .= ">".$name[$idx]."</OPTION>\n";
			} // END - foreach
		} // END - if
	} else {
		// Data from database
		$SPEC = ", ".$id;
		if (!empty($special)) $SPEC = ", ".$special;
		$ORDER = $name.$SPEC;
		if ($table == "country") $ORDER = $special;
		$result = SQL_QUERY_ESC("SELECT %s, %s".$SPEC." FROM "._MYSQL_PREFIX."_%s ".$where." ORDER BY %s",
		 array($id, $ORDER, $table, $name), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) > 0) {
			// Found data so add them as OPTION lines: $id is the value and $name is the "name" of the option
			while (list($value, $title, $add) = SQL_FETCHROW($result)) {
				if (empty($special)) $add = "";
				$ret .= "<OPTION value=\"".$value."\"";
				if ($default == $value) $ret .= " selected checked";
				if (!empty($add)) $add = " (".$add.")";
				$ret .= ">".$title.$add."</OPTION>\n";
			} // END - while

			// Free memory
			SQL_FREERESULT($result);
		} else {
			// No data found
			$ret = "<OPTION value=\"x\">".SELECT_NONE."</OPTION>\n";
		}
	}

	// Return - hopefully - the requested data
	return $ret;
}
// Activate exchange (DEPERECATED???)
function activateExchange() {
	global $_CONFIG;
	$result = SQL_QUERY("SELECT userid FROM "._MYSQL_PREFIX."_user_data WHERE status='CONFIRMED' AND max_mails > 0", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) >= $_CONFIG['activate_xchange'])
	{
		// Free memory
		SQL_FREERESULT($result);

		// Activate System
		$SQLs = array(
			"UPDATE "._MYSQL_PREFIX."_mod_reg SET locked='N', hidden='N', mem_only='Y' WHERE module='order' LIMIT 1",
			"UPDATE "._MYSQL_PREFIX."_member_menu SET visible='Y', locked='N' WHERE what='order' OR what='unconfirmed' LIMIT 2",
			"UPDATE "._MYSQL_PREFIX."_config SET activate_xchange='0' WHERE config=0 LIMIT 1"
		);

		// Run SQLs
		foreach ($SQLs as $sql) {
			$result = SQL_QUERY($sql, __FILE__, __LINE__);
		}

		// @TODO Destroy cache
	}
}
//
function DELETE_USER_ACCOUNT($uid, $reason)
{
	$points = 0;
	$result = SQL_QUERY_ESC("SELECT (SUM(p.points) - d.used_points) AS points
FROM "._MYSQL_PREFIX."_user_points AS p
LEFT JOIN "._MYSQL_PREFIX."_user_data AS d
ON p.userid=d.userid
WHERE p.userid=%s", array(bigintval($uid)), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 1) {
		// Save his points to add them to the jackpot
		list($points) = SQL_FETCHROW($result);
		SQL_FREERESULT($result);

		// Delete points entries as well
		$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_user_points WHERE userid=%s", array(bigintval($uid)), __FILE__, __LINE__);

		// Update mediadata as well
		if (GET_EXT_VERSION("mediadata") >= "0.0.4") {
			// Update database
			MEDIA_UPDATE_ENTRY(array("total_points"), "sub", $points);
		} // END - if

		// Now, when we have all his points adds them do the jackpot!
		ADD_JACKPOT($points);
	}

	// Delete category selections as well...
	$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_user_cats WHERE userid=%s",
	 array(bigintval($uid)), __FILE__, __LINE__);

	// Remove from rallye if found
	if (EXT_IS_ACTIVE("rallye")) {
		$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_rallye_users WHERE userid=%s",
		 array(bigintval($uid)), __FILE__, __LINE__);
	}

	// Now a mail to the user and that's all...
	$msg = LOAD_EMAIL_TEMPLATE("del-user", array('text' => $reason), $uid);
	SEND_EMAIL($uid, ADMIN_DEL_ACCOUNT, $msg);

	// Ok, delete the account!
	$result = SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_user_data WHERE userid=%s LIMIT 1", array(bigintval($uid)), __FILE__, __LINE__);
}
//
function META_DESCRIPTION($mod, $wht)
{
	global $_CONFIG, $DEPTH;
	if (($mod != "admin") && ($mod != "login"))
	{
		// Exclude admin and member's area
		$DESCR = MAIN_TITLE." ".trim($_CONFIG['title_middle'])." ".ADD_DESCR("guest", "what-".$wht, true);
		unset($DEPTH);
		OUTPUT_HTML("<META name=\"description\" content=\"".$DESCR."\">");
	}
}
//
function ADD_JACKPOT($points)
{
	$result = SQL_QUERY("SELECT points FROM "._MYSQL_PREFIX."_jackpot WHERE ok='ok' LIMIT 1", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 0)
	{
		// Create line
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_jackpot (ok, points) VALUES ('ok','%s')", array($points), __FILE__, __LINE__);
	}
	 else
	{
		// Free memory
		SQL_FREERESULT($result);

		// Update points
		$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_jackpot SET points=points+%s WHERE ok='ok' LIMIT 1",
		 array($points), __FILE__, __LINE__);
	}
}
//
function SUB_JACKPOT($points)
{
	// First failed
	$ret = "-1";

	// Get current points
	$result = SQL_QUERY("SELECT points FROM "._MYSQL_PREFIX."_jackpot WHERE ok='ok' LIMIT 1", __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 0)
	{
		// Create line
		$result = SQL_QUERY("INSERT INTO "._MYSQL_PREFIX."_jackpot (ok, points) VALUES ('ok', 0.00000)", __FILE__, __LINE__);
	}
	 else
	{
		// Free memory
		SQL_FREERESULT($result);

		// Read points
		list($jackpot) = SQL_FETCHROW($result);
		if ($jackpot >= $points)
		{
			// Update points when there are enougth points in jackpot
			$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_jackpot SET points=points-%s WHERE ok='ok' LIMIT 1",
				array($points), __FILE__, __LINE__);
			$ret = $jackpot - $points;
		}
	}
}
//
function IS_DEMO() {
	return ((EXT_IS_ACTIVE("demo")) && (get_session('admin_login') == "demo"));
}
//
function LOAD_CONFIG($no="0") {
	global $cacheArray;
	$CFG_DUMMY = array();

	// Check for cache extension, cache-array and if the requested configuration is in cache
	if ((is_array($cacheArray)) && (isset($cacheArray['config'][$no])) && (is_array($cacheArray['config'][$no]))) {
		// Load config from cache
		//* DEBUG: */ echo gettype($cacheArray['config'][$no])."<br />\n";
		foreach ($cacheArray['config'][$no] as $key => $value) {
			$CFG_DUMMY[$key] = $value;
		} // END - foreach

		// Count cache hits if exists
		if ((isset($CFG_DUMMY['cache_hits'])) && (EXT_IS_ACTIVE("cache"))) {
			$CFG_DUMMY['cache_hits']++;
		} // END - if
	} else {
		// Load config from DB
		$result_config = SQL_QUERY_ESC("SELECT * FROM "._MYSQL_PREFIX."_config WHERE config=%d LIMIT 1",
			array(bigintval($no)), __FILE__, __LINE__);

		// Get config from database
		$CFG_DUMMY = SQL_FETCHARRAY($result_config);

		// Free result
		SQL_FREERESULT($result_config);

		// Remember this config in the array
		$cacheArray['config'][$no] = $CFG_DUMMY;
	}

	// Return config array
	return $CFG_DUMMY;
}
// Gets the matching what name from module
function GET_WHAT($MOD_CHECK) {
	global $_CONFIG;

	$wht = "";
	//* DEBUG: */ echo __LINE__."!".$MOD_CHECK."!<br />\n";
	switch ($MOD_CHECK)
	{
	case "admin":
		$wht = "overview";
		break;

	case "login":
	case "index":
		$wht = "welcome";
		if (($MOD_CHECK == "index") && (!empty($_CONFIG['index_home']))) $wht = $_CONFIG['index_home'];
		break;

	default:
		$wht = "";
		break;
	}

	// Return what value
	return $wht;
}
//
function MODULE_HAS_MENU($mod, $forceDb = false)
{
	global $cacheArray, $_CONFIG;

	// All is false by default
	$ret = false;
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):mod={$mod},cache=".GET_EXT_VERSION("cache")."<br />\n";
	if (GET_EXT_VERSION("cache") >= "0.1.2") {
		// Cache version is okay, so let's check the cache!
		if (isset($cacheArray['modules']['has_menu'][$mod])) {
			// Check module cache and count hit
			$ret = ($cacheArray['modules']['has_menu'][$mod] == "Y");
			$_CONFIG['cache_hits']++;
		} elseif (isset($cacheArray['extensions']['ext_menu'][$mod])) {
			// Check cache and count hit
			$ret = ($cacheArray['extensions']['ext_menu'][$mod] == "Y");
			$_CONFIG['cache_hits']++;
		}
	} elseif ((GET_EXT_VERSION("sql_patches") >= "0.3.6") && ((!EXT_IS_ACTIVE("cache")) || ($forceDb === true))) {
		// Check database for entry
		$result = SQL_QUERY_ESC("SELECT has_menu FROM "._MYSQL_PREFIX."_mod_reg WHERE module='%s' LIMIT 1",
		 array($mod), __FILE__, __LINE__);
		if (SQL_NUMROWS($result) == 1) {
			list($has_menu) = SQL_FETCHROW($result);

			// Fake cache... ;-)
			$cacheArray['extensions']['ext_menu'][$mod] = $has_menu;

			// Does it have a menu?
			$ret = ($has_menu == "Y");
		} // END  - if

		// Free memory
		SQL_FREERESULT($result);
	} elseif (GET_EXT_VERSION("sql_patches") == "") {
		// No sql_patches installed, so maybe in admin area?
		$ret = ((IS_ADMIN()) && ($mod == "admin")); // Then there is a menu!
	}

	// Return status
	return $ret;
}
// Subtract points from database and mediadata cache
function SUB_POINTS ($uid, $points) {
	// Add points to used points
	$result = SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_user_data SET `used_points`=`used_points`+%s WHERE userid=%s LIMIT 1",
	 array($points, bigintval($uid)), __FILE__, __LINE__);

	// Update mediadata as well
	if (GET_EXT_VERSION("mediadata") >= "0.0.4") {
		// Update database
		MEDIA_UPDATE_ENTRY(array("total_points"), "sub", $points);
	} // END - if
}
// Update config entries
function UPDATE_CONFIG ($entries, $values, $updateMode="") {
	// Do we have multiple entries?
	if (is_array($entries)) {
		// Walk through all
		$all = "";
		foreach ($entries as $idx => $entry) {
			// Update mode set?
			if (!empty($updateMode)) {
				// Update entry
				$all .= sprintf("%s=%s%s%s,", $entry, $entry, $updateMode, (float)$values[$idx]);
			} else {
				// Check if string or number
				if (($values[$idx] + 0) === $values[$idx]) {
					// Number detected
					$all .= sprintf("%s=%s,", $entry, (float)$values[$idx]);
				} else {
					// String detected
					$all .= sprintf("%s='%s',", $entry, SQL_ESCAPE($values[$idx]));
				}
			}
		} // END - foreach

		// Remove last comma
		$entries = substr($all, 0, -1);
	} elseif (!empty($updateMode)) {
		// Update mode set
		$entries .= sprintf("=%s%s%s", $entries, $updateMode, (float)$value);
	} else {
		// Regular entry to update
		$entries .= sprintf("='%s'", SQL_ESCAPE($values));
	}

	// Run database update
	//DEBUG_LOG(__FUNCTION__.":entries={$entries}");
	SQL_QUERY("UPDATE "._MYSQL_PREFIX."_config SET ".$entries." WHERE config=0 LIMIT 1", __FILE__, __LINE__);

	// Get affected rows
	$affectedRows = SQL_AFFECTEDROWS();
	//* DEBUG: */ print __FUNCTION__."(<font color=\"#0000aa\">".__LINE__."</font>):entries={$entries},affectedRows={$affectedRows}<br />\n";

	// Rebuild cache
	REBUILD_CACHE("config", "config");
}
// Creates a new task for updated extension
function CREATE_EXTENSION_UPDATE_TASK ($admin_id, $subject, $notes) {
	// Check if task is not there
	$result = SQL_QUERY_ESC("SELECT id FROM "._MYSQL_PREFIX."_task_system WHERE subject='%s' LIMIT 1",
		array($subject), __FILE__, __LINE__);
	if (SQL_NUMROWS($result) == 0) {
		// Task not created so it's a brand-new extension which we need to register and create a task for!
		$result = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_task_system (assigned_admin, userid, status, task_type, subject, text, task_created) VALUES ('%s','0','NEW','EXTENSION_UPDATE','%s','%s', UNIX_TIMESTAMP())",
			array($admin_id, $subject, $notes), __FILE__, __LINE__);
	} // END - if

	// Free memory
	SQL_FREERESULT($result);
}
// Creates a new task for newly installed extension
function CREATE_NEW_EXTENSION_TASK ($admin_id, $subject, $ext) {
	// Not installed and do we have created a task for the admin?
	$result = SQL_QUERY_ESC("SELECT `id` FROM `"._MYSQL_PREFIX."_task_system` WHERE `subject` LIKE '%s%%' LIMIT 1",
		array($subject), __FILE__, __LINE__);
	if ((SQL_NUMROWS($result) == 0) && (GET_EXT_VERSION($ext) == "")) {
		// Template file
		$tpl = sprintf("%stemplates/%s/html/ext/ext_%s.tpl",
			PATH,
			GET_LANGUAGE(),
			$ext
		);

		// Load text for task
		if (FILE_READABLE($tpl)) {
			// Load extension's own text template (HTML!)
			$msg = LOAD_TEMPLATE("ext_".$ext, true);
		} else {
			// Load default message
			$msg = LOAD_TEMPLATE("admin_new_ext", "", 0);
		}

		// Task not created so it's a brand-new extension which we need to register and create a task for!
		$result_insert = SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_task_system (assigned_admin, userid, status, task_type, subject, text, task_created)
VALUES (%s,0,'NEW','EXTENSION','%s','%s',UNIX_TIMESTAMP())",
			array(
				$admin_id,
				$subject,
				addslashes($msg),
			),  __FILE__, __LINE__, true, false
		);
	} // END - if

	// Free memory
	SQL_FREERESULT($result);
}
//
?>
